#!/bin/bash
# Script to split installation into tarball feeds
# - base  : items that must always be installed
# - dev   : items for development: header files, static libraries, dynamic libraries links
# - dbg   : items for delivery with debug feature unstripped
# - doc   : items for delivery with generated documentations
# - locale: items related to internationalization

: ${debug_package_split:=false}

if [ -r "$PLATFORM_LOC/scripts/utils.sh" ]; then
	source "$PLATFORM_LOC/scripts/utils.sh"
else
	source "$script_lib_dir/utils.sh"
fi
$debug_package_split && print_traces split
split_dbg() {
	dbg split "$@"
}
split_dbg_cat() {
	dbg_cat split "$@"
}
split_dbg "Starting split step..."

# Give default values to parameters (:= if null or unset, = if only unset, variables must be defined)

: ${splitted_packages:=$package_name}
: ${sub_packages:="doc config locale i18n shlibs staticdev dev dbg test base"}
: ${needed_subpackages:="staticdev dev dbg base"}
: ${installDir:=$here/_install}
: ${unsplitted_shlibs:=libc libm}

set -o noglob
: ${pattern_base:="/bin /sbin /usr/bin /usr/sbin /usr/libexec /usr/share"}
: ${pattern_config:="/etc /usr/etc /usr/*/etc"}
: ${pattern_shlibs:="/lib/*.so /usr/lib/*.so /lib/*.so-* /usr/lib/*.so-* /lib/*.so.* /usr/lib/*.so.*"}
: ${pattern_locale:="/usr/share/locale"}
: ${pattern_i18n:="/usr/share/i18n"}
: ${pattern_staticdev:="/lib/*.a /usr/lib/*.a /usr/*/lib/*.a"}
: ${pattern_dev:="/lib/*.o /usr/lib/*.o /usr/*/lib/*.o /lib/*.la /usr/lib/*.la /usr/*/lib/*.la /usr/include /lib/pkgconfig /usr/lib/pkgconfig /usr/share/pkgconfig /usr/share/aclocal"}
: ${pattern_dbg:="/bin/.debug /lib/.debug /usr/bin/.debug /usr/lib/.debug"}
: ${pattern_doc:="/usr/share/doc /usr/share/man /usr/share/info"}
set +o noglob

# Note tempDir, like any other xxxDir must be an absolute path 
tempDir=/tmp/$package_name.$$
trap "rm -rf $tempDir" EXIT
rm -rf $tempDir

mkdir -p $tempDir/install
tar cf - -C $installDir . | tar xf - -C $tempDir/install
cd $tempDir/install
# Remove the fakeroot state, as it is not part of the package...
rm -f .fakeroot

# Generate in a temporary directory, before moving in feedsDir
metadataTempDir=$tempDir/feeds/$metadataSubDir
contentsTempDir=$tempDir/feeds/$contentsSubDir
packagesTempDir=$tempDir/feeds/$packagesSubDir
tarballsTempDir=$tempDir/feeds/$tarballsSubDir

mkdir -p $contentsTempDir $packagesTempDir $tarballsTempDir $metadataTempDir
allContentsDir=$feedsDir/contents/$package_arch

do_buildstep_prepend

base_bin_packages=
forced_bin_packages=
other_bin_packages=

declare -A subpkgs # Subpackages declared for a given base package
declare -A basepkg # Base package of a given subpackage
declare -A pkgkind # Kind (i.e. subpkg) of a given package
declare -A pkgname # Variable name of the package

# Generate list of packages and package content

check_package_base() {
	local pkg="$1"
	local isbase=true
	local base sub
	
	if [ "$pkg" != "$package_name" -a "${pkg%-*}" != "$pkg" ]; then
		for sub in $sub_packages; do
			if [ $sub = "base" ]; then continue; fi
			base=${pkg%-$sub}
			if [ "$base" = "$pkg" ]; then continue; fi
			split_dbg "Record '$pkg' is a sub-package of '$base'"
			subpkgs[$base]="${subpkgs[$base]} $sub"
			basepkg[$pkg]="$base"
			pkgkind[$pkg]="$sub"
			isbase=false
			split_dbg "Found '$base' as base package for '$pkg' (1)"
			break
		done
		if $isbase; then
			# Check if there is another package with the same "base" root
			base=$pkg
			while true; do
				sub=${base%-*} # Suppress the last dash-delimited component
				if [ "$sub" = "$base" ]; then break; fi
				base=$sub
				if [[ " $splitted_packages $package_name " =~ " $base " ]]; then
					# This is a non-standard sub-package of a defined base package
					sub=${pkg#$base-}
					isbase=false
					split_dbg "Found '$base' as base package for '$pkg' (2)"
					break
				elif [[ " $splitted_packages $package_name " =~ " $base-" ]]; then
					# This maybe a non-standard sub-package of a possibly not yet knowned base package
					for sub in $splitted_packages $package_name; do
						if [ "$sub" = "$pkg" ]; then continue; fi
						if [[ " $sub " =~ " $base-" ]]; then
							# There are several sub-packages of base, so go ahead		
							sub=${pkg#$base-}
							isbase=false
							split_dbg "Found '$base' as base package for '$pkg' (3)"
							break
						fi
					done
				fi
				$isbase || break
			done
		fi
#		if $isbase; then
#			# Check now if there is an existing base package for which we could be a subpackage
#			for base in ${!subpkgs[@]}; do
#				if [[ " $pkg " =~ " $base-" ]]; then
#					sub=${pkg#$base-}
#					isbase=false
#					split_dbg "Found '$base' as possible base package fo '$pkg'"
#					break
#				fi
#			done
#		fi
#		if $isbase; then
#			sub=${pkg#${package_name}-}
#			if [ "$sub" != "$pkg" ]; then
#				# This is a non-standard sub-package of main package
#				base=${pkg%-$sub}
#				isbase=false
#				split_dbg "Found '$base' as possible base package fo '$pkg'"
#			fi
#		fi
	fi
	if $isbase; then
		# This is really a plain base package
		subpkgs[$pkg]="${subpkgs[$pkg]} base"
		basepkg[$pkg]=
		pkgkind[$pkg]="base"
	else
		if [ -z "${subpkgs[$base]}" ]; then
			split_dbg "Seems that '$pkg' is a sub-package of '$base'; check it is a base..."
			check_package_base $base
		fi
		# Now record its new sub-package
		subpkgs[$base]="${subpkgs[$base]} $sub"
		basepkg[$pkg]="$base"
		pkgkind[$pkg]="$sub"
	fi

	# Debug trace
	if $isbase; then
		split_dbg ">>>>>> $pkg"
	else
		split_dbg "       $pkg - $base"
	fi
	# End debug trace

	var_name=$(shell_variable_name $pkg)
	pkgname[$pkg]="$var_name"
}

if [ "$splitted_packages" = "$package_name" -o -z "$splitted_packages" ]; then
	var_name=$(shell_variable_name $package_name)
	splitted_packages=
	for pkg in $sub_packages; do
		if [ "$pkg" = "base" ]; then
			splitted_packages="${splitted_packages# } $package_name"
			subpkgs[$package_name]="${subpkgs[$package_name]# } base"
			basepkg[$package_name]=
			pkgkind[$package_name]="base"
			pkgname[$package_name]="$var_name"
		else
			splitted_packages="${splitted_packages# } $package_name-$pkg"
			subpkgs[$package_name]="${subpkgs[$package_name]# } $pkg"
			basepkg[$package_name-$pkg]="$package_name"
			pkgkind[$package_name-$pkg]="$pkg"
			pkg_name=${pkg//+/P}
			pkgname[$package_name-$pkg]="$(shell_variable_name $package_name-$pkg)"
		fi
	done
else
	for pkg in $splitted_packages; do
		check_package_base $pkg
	done
fi
all_packages=" $splitted_packages "

#
# Add the missing needed base or sub packages (will be generated as empty)
#

for base in ${!subpkgs[@]}; do
	# Base is checked first so that if we need a subpackage, the base is there already
	for needed in "base" $needed_subpackages; do
		if [[ " ${subpkgs[$base]} " =~ " $needed " ]]; then
			continue
		fi
		if [ "$needed" = "base" ]; then 
			# Add it after the last sub-package
			split_dbg "Add missing base package '$base' due to subpackages '${subpkgs[$base]}'"
			lastsub=${subpkgs[$base]##* }
			all_packages=${all_packages/ $base-$lastsub / $base-$lastsub $base }
			subpkgs[$base]="${subpkgs[$base]} base"
			basepkg[$base]=
			pkgkind[$base]="base"
			var_name=$(shell_variable_name $base)
			pkgname[$base]="$var_name"
			split_dbg "Variable name for '$base' is '$var_name'"
		else
			# Add it before its base
			split_dbg "Add missing subpackage '$base-$needed'"
			all_packages=${all_packages/ $base / $base-$needed $base }
			subpkgs[$base]="${subpkgs[$base]} $needed"
			basepkg[$base-$needed]="$base"
			pkgkind[$base-$needed]="$needed"
			var_name=$(shell_variable_name $base-$needed)
			pkgname[$base-$needed]="$var_name"
			split_dbg "Variable name for '$base-$needed' is '$var_name'"
		fi
	done
done

duplicate_metadata() {
	local base_package="$1"
	local new_package="$2"
	local message="$3"

	for spec in SUMMARY DESCRIPTION SECTION HOMEPAGE LICENSE RSUGGESTS RCONFLICTS RREPLACES; do
		if [ -r "$metadataTempDir/$base_package.${spec,,}" -a ! -r "$metadataTempDir/$new_package.${spec,,}" ]; then
			if [ $spec = DESCRIPTION ]; then
				cp $metadataTempDir/$base_package.${spec,,} $metadataTempDir/$new_package.${spec,,}
				echo "This package contains the $message." >> $metadataTempDir/$new_package.${spec,,}
			elif [ $spec = SUMMARY ]; then
				echo "${message^}. " > $metadataTempDir/$new_package.${spec,,}
				cat $metadataTempDir/$base_package.${spec,,} >> $metadataTempDir/$new_package.${spec,,}
			else
				cp $metadataTempDir/$base_package.${spec,,} $metadataTempDir/$new_package.${spec,,}
			fi
		fi
	done
}

build_package() {
	local bin_package="$1"
	local file_list="$2"
	local package="$3"
	local pkg="$4"
	local pkgTarball=${bin_package}.tar.bz2

	if [ -s "$file_list" ]; then
		log "Generating ${package} ${pkg^^} package feed: '$bin_package'"
	else
		log "Force generation of empty ${package} ${pkg^^} package feed: '$bin_package'"
	fi
	
	log "tar -cjf '$tarballsTempDir/$pkgTarball' --files-from='$file_list'"
	tar -cjf "$tarballsTempDir/$pkgTarball" --files-from="$file_list"
	echo "$package_id" > "$packagesTempDir/$bin_package"
	touch "$metadataTempDir/$recipe_id.manifest"
	echo "$bin_package" >> "$metadataTempDir/$recipe_id.manifest"

	# Remove the packaged files so that a file is only packaged once
	for i in $(cat $file_list); do
		rm -f $i
	done
}

print_dependencies() {
	local bin_package="$1"
	local package_name=${bin_package%%_*}
	local package_vers=${bin_package#*_}
	shift
	
	local kind="$1"
	local ekind
	shift
	
	local subpkgs="$1"
	shift
	
	local rdeps="$*"
	split_dbg "Printing package dependencies:"
	split_dbg "    Package:      $bin_package = ($package_name - $package_vers)"
	split_dbg "    Kind:         $kind"
	split_dbg "    Subpackages:  $subpkgs"
	split_dbg "    Dependencies: $rdeps"
	
	local subpkg pkg pkgsub rdep rdep_name rdep_vers sep
	
	for subpkg in $subpkgs; do
		split_dbg "Subpackage: $subpkg"
		if [ "$subpkg" = "base" ]; then
			subpkg=""
		else
			subpkg="-$subpkg"
		fi
		pkg=${package_name}${subpkg}_${package_vers}
		split_dbg "    Dependent package: $pkg"
		pkgsub="${pkgkind[${package_name}${subpkg}]}"
		if [ -z "${pkgsub}" ]; then
			split_dbg "        Split non-existent package"
			continue # Non-existent package... skip
		fi
		split_dbg "    Subpackage kind: $pkgsub"
		
		if [ "$pkgsub" = "base" -o "$pkgsub" = "shlibs" ]; then
			ekind=$kind
			pkgsub=""
		else
			if [[ " $needed_subpackages " =~ " $pkgsub " ]]; then
				ekind=$kind
			else
				if [ "$kind" = "rdepends" ]; then
					# Propagate dependencies as recommendations for non-needed subpackages
					ekind="rrecommends"
				else
					ekind=$kind
				fi
			fi
			pkgsub="-$pkgsub"
		fi
		split_dbg "    Rewritten subpackage kind: $pkgsub - $ekind"
	
		split_dbg "    Adding dependents to '$pkg.$ekind'"
		touch $metadataTempDir/$pkg.$ekind
		opened_line=false
		for rdep in $rdeps; do
			sep=
			case $rdep in
				[a-zA-Z0-9]*)
					if $opened_line; then
						echo ''
						sep=
					fi
					rdep_name=${rdep%%_*}
					rdep_vers=${rdep#*_}
					if [ "$rdep_vers" = "$rdep_name" ]; then
						rdep_vers=
					else
						rdep_vers="_$rdep_vers"
					fi
					split_dbg "        ${rdep_name}${pkgsub}${rdep_vers}"
					echo -n "${sep}${rdep_name}${pkgsub}${rdep_vers}" 
					sep=' '
					opened_line=true
					;;
				\() echo -n ' ('
					;;
				\))	echo -n ')'
					;;
				*)	echo -n " $rdep"
					;;
			esac
		done >> $metadataTempDir/$pkg.$ekind
		if $opened_line; then echo '' >> $metadataTempDir/$pkg.$ekind; fi
		split_dbg_cat "Current ${subpkg^^} metadata for '$pkg':"  $metadataTempDir/$pkg.$ekind
	done
}

empty=
for package in $all_packages; do
	pkg=${pkgkind[$package]}
	base=${basepkg[$package]}
	bin_package=${package}_${package_suffix}
	split_dbg "Generating '$bin_package'"
	
	forced=true
	if [ "$pkg" = "base" -o "$pkg" = "shlibs" ]; then
		base_bin_packages="$base_bin_packages $bin_package"
	elif [[ " $needed_subpackages " =~ " $pkg " ]]; then
		forced_bin_packages="$forced_bin_packages $bin_package"
	elif [[ " $needed_subpackages " =~ " $package " ]]; then
		forced_bin_packages="$forced_bin_packages $bin_package"
	else
		other_bin_packages="$other_bin_packages $bin_package"
		forced=false
	fi

	# Create a file with the list of files to put in the archive
	# This has the additional benefit of allowing to create empty archives if needed
	file_list=$contentsTempDir/$bin_package.list
	pattern_name=pattern_${pkgname[$package]}
	split_dbg "Looking for pattern '$pattern_name'"
	pattern_set="${!pattern_name+set}"
	if [ -z "$pattern_set" ]; then
		split_dbg "Pattern '$pattern_name' not set"
		if [ -z "$base" ]; then
			suffix="base"
			pattern_name=pattern_base
		elif [ "$base" = "$package_name" ]; then
			suffix=${pkgname[$package]#${pkgname[$base]}_}
			pattern_name=pattern_${suffix:-"base"}
		fi
		split_dbg "	using '$pattern_name' instead"
	fi

	# We add shlibs in the main pattern, so that non-splitted shlibs are always loaded
	shlibs_pattern_name="${pattern_name%_base}_shlibs"
	pattern_val="${!pattern_name}"
	shlibs_pattern_val="${!shlibs_pattern_name}"
	split_dbg "pattern_val='$pattern_val'"
	[ -n "$shlibs_pattern_val" ] && split_dbg "shlibs_pattern_val='$shlibs_pattern_val'"
	if [ -n "$pattern_val" -o -n "$shlibs_pattern_val" ]; then
		pattern=
		# Transform paths to local paths in copied install directory...
		set -o noglob
		for fspec in $pattern_val $shlibs_pattern_val; do
			pattern="${pattern# } ${fspec#/}"
		done
		split_dbg "pattern='$pattern'"
		set +o noglob
		for fspec in $(echo ${pattern}); do
			if [ -d $fspec ]; then
				find $fspec ! -type d
			elif [ -e $fspec -o -L $fspec ]; then
				echo $fspec
			fi
		done
	fi | sort | uniq > $file_list
	split_dbg_cat "File list in" "$file_list"
	
	if [ -s "$file_list" ] || $forced; then
		true # generate the package
	else
		continue # Skip empty package
	fi

	# Generate the package description metadata and dependencies
	for spec in SUMMARY DESCRIPTION SECTION HOMEPAGE LICENSE RDEPENDS RRECOMMENDS RSUGGESTS RCONFLICTS RREPLACES; do
		spec_var=${spec}_${pkgname[$package]}
		val_set=${!spec_var+set}
		if [ -z "$val_set" ]; then
			if [ "$pkg" != "base" ]; then
				base=${pkgbase[$package]}
				if [ -n "$base" ]; then
					suffix=${pkgname[$package]#${pkgname[$base]}_}
					if [ "$base" = "$package_name" ]; then
						spec_var=${spec}_$suffix
						val_set=${!spec_var+set}
					fi
					if [ -z "$val_set" ]; then
						spec_var=${spec}_${pkgname[$base]}
						val_set=${!spec_var+set}
					fi
					if [ "$base" != "$package_name" ]; then
						if [ -z "$val_set" ]; then
							spec_var=${spec}_${pkgname[$package_name]}_$suffix
							val_set=${!spec_var+set}
						fi
						if [ -z "$val_set" ]; then
							spec_var=${spec}_$suffix
							val_set=${!spec_var+set}
						fi
					fi
				else
					if [ -z "$val_set" ]; then
						spec_var=${spec}_${pkgname[$package_name]}_$suffix
						val_set=${!spec_var+set}
					fi
					if [ -z "$val_set" ]; then
						spec_var=${spec}_$suffix
						val_set=${!spec_var+set}
					fi
				fi
			else
				spec_var=${spec}_${pkgname[$package]}_base
				val_set=${!spec_var+set}
			fi
		fi
		if [ -z "$val_set" ]; then
			spec_var=${spec}_${pkgname[$package_name]}
			val_set=${!spec_var+set}
		fi
		if [ -z "$val_set" ]; then
			spec_var=${spec}_${pkgname[$package_name]}_base
			val_set=${!spec_var+set}
		fi
		if [ -z "$val_set" ]; then
			spec_var=${spec}_base
			val_set=${!spec_var+set}
		fi
		if [ "$val_set" = "set" ]; then
			case $spec in
				RDEPENDS | RRECOMMENDS | RSUGGESTS | RCONFLICTS | RREPLACES)
					print_dependencies $bin_package ${spec,,} "$needed_subpackages" ${!spec_var}
					;;
				*)
					echo "${!spec_var}" > $metadataTempDir/$bin_package.${spec,,}
					;;
			esac
		fi
		split_dbg_cat "Current '${spec^^} metadata for '$bin_package':" $metadataTempDir/$bin_package.${spec,,}
	done

	# Don't forget to add the recipe runtime dependencies to the generated package
	print_dependencies $bin_package rdepends "base" ${recipe_runtime_dependencies}

	if [ "$pkg" = "shlibs" ]; then
		# Here we must generate the splitted shlibs packages, 
		# as well as their dev and dbg partners
		shlibs=$(sed -n -e '/\.so$/s|^.*/\(lib[A-Za-z0-9+]\+\)[-_.].*\.so$|\1|p'				\
		                -e '/\.so$/s|^.*/\(lib[A-Za-z0-9+]\+\)\.so$|\1|p'						\
		                -e '/\.so[-.][0-9].*$/s|^.*/\(lib[A-Za-z0-9+]\+\).*\.so[-.].*$|\1|p'	\
		                $file_list | sort | uniq)
		split_dbg "shlibs='$shlibs'"
		
		all_shlibs_pkgs=
		for shlib in ${shlibs}; do
			if [[ " ${unsplitted_shlibs} " =~ " ${shlib} " ]]; then
				split_dbg "Unsplitted shlib '$shlib'"
				continue # Keep these in the main package
			fi
			if [ " ${shlib} " = " ${package_name} " -o " ${shlib} " = " lib${package_name} " ]; then
				split_dbg "Main shlib '$shlib'"
				continue # This is the main package
			fi

			# Do not try to generate if an explicit package was defined
			shlib_base="${package/shlibs/${shlib}}"
			if [ -n "${pkgkind[${shlib_base}]}" ]; then
				split_dbg "Package '$shlib_base' already defined"
				continue # Anyway this package should be empty (or the explicit package will be incomplete...)
			fi
		
			#
			# Generate list of all files for this shared library
			#
			full_list=${file_list}.${shlib}-full
			grep -e "/${shlib}[-._]" $file_list | sort | uniq > "${full_list}"
			split_dbg_cat "Full shlib list in" "${full_list}"
		
			#
			# Shared objects 
			#
			# This includes all files matched by shutils except symbolic links in *.so
			#
			so_list=${file_list/shlibs/${shlib}}
			grep -v "/${shlib}.so\$/" ${full_list} > ${so_list}
			for sofile in $(grep "/${shlib}.so\$/" ${full_list}); do
				if [ ! -L $sofile ]; then
					echo $sofile
				fi
			done >> ${so_list}
			if [ ! -s "${so_list}" ]; then
				rm -f ${full_list}
				split_dbg "No shared objects for '$shlib', skip entirely; files will go in the main shlibs package"
				continue # No shared objects, skip entirely; files will go in the main shlibs package
			fi
			split_dbg_cat "Shared objects list in" "${so_list}"
			# This is a new base package:
			shlib_pkg="${shlib_base}"
			subpkgs[${shlib_base}]="${subpkgs[${shlib_base}]} base"
			basepkg[${shlib_pkg}]="${package}"
			pkgkind[${shlib_pkg}]="shlibs"
			# Generate it
			bin_shlib_pkg=${shlib_pkg}_${package_suffix}
			split_dbg "bin_shlib_pkg=$bin_shlib_pkg"
			duplicate_metadata ${bin_package} ${bin_shlib_pkg} "shared objects for ${shlib}"
			build_package ${bin_shlib_pkg} ${so_list} ${shlib} base
			base_bin_packages="${base_bin_packages} ${bin_shlib_pkg}"
			# Eliminate packaged files from full list
			mv "$full_list" "$full_list.bak"
			comm -13 "${so_list}" "$full_list.bak" > "$full_list"
			rm -f "$full_list.bak"
			# Record it in the list of shared library packages
			all_shlibs_pkgs="$all_shlibs_pkgs $bin_shlib_pkg"
			
			#
			# Development files
			#
			dev_list=${file_list/shlibs/${shlib}-dev}
			# Here I must also add the appropriate .so, .o and .la files, matching elements from the -dev subpkg,
			# as well as popssible _nonshared.a files...
			(
				grep "/${shlib}.so\$/" ${full_list} > ${dev_list}
				for dir in lib usr/lib usr/*/lib; do
					if [ -d "$dir" ]; then 
						find $dir -name "${shlib}.so"
						find $dir -name "${shlib}*.o"
						find $dir -name "${shlib}*.la"
						find $dir -name "${shlib}*.a"
					fi
				done
			) | sort | uniq > ${dev_list}
			split_dbg_cat "Development shlib links in" "${dev_list}"
			# This is a new derived package:
			shlib_pkg="${package/shlibs/${shlib}-dev}"
			subpkgs[${shlib_base}]="${subpkgs[${shlib_base}]} dev"
			basepkg[${shlib_pkg}]="${shlib_base}"
			pkgkind[${shlib_pkg}]="dev"
			# Generate it
			bin_shlib_pkg=${shlib_pkg}_${package_suffix}
			duplicate_metadata ${bin_package} ${bin_shlib_pkg} "development files for ${shlib}"
			build_package ${bin_shlib_pkg} ${dev_list} ${shlib} dev
			forced_bin_packages="${forced_bin_packages} ${bin_shlib_pkg}"
			# Eliminate packaged files from full list
			mv "$full_list" "$full_list.bak"
			comm -13 "${dev_list}" "$full_list.bak" > "$full_list"
			rm -f "$full_list.bak"
			
			#
			# Debug info
			#
			dbg_list=${file_list/shlibs/${shlib}-dbg}
			for sofile in $(cat ${so_list} ${dev_list}); do
				dbgfile=${sofile/\/lib${shlib}/.debug/lib${shlib}}
				if [ -r ${dbgfile} ]; then
					echo ${dbgfile}
				fi
			done | sort | uniq > ${dbg_list}
			split_dbg_cat "Debug shlib components in" "${dbg_list}"
			# This is a new derived package:
			shlib_pkg="${package/shlibs/${shlib}-dbg}"
			subpkgs[${shlib_base}]="${subpkgs[${shlib_base}]} dbg"
			basepkg[${shlib_pkg}]="${shlib_base}"
			pkgkind[${shlib_pkg}]="dbg"
			# Generate it
			bin_shlib_pkg=${shlib_pkg}_${package_suffix}
			duplicate_metadata $bin_package ${bin_shlib_pkg} "debug symbols for ${shlib}"
			build_package ${bin_shlib_pkg} ${dbg_list} ${shlib} dbg
			forced_bin_packages="${forced_bin_packages} ${bin_shlib_pkg}"
			
			# Now filter out the shlib from the global file
			cat ${so_list} ${dev_list} ${staticdev_list} ${dbg_list} | sort | uniq > ${full_list}
			sort ${file_list} | uniq > ${file_list}.sorted
			comm -13 ${full_list} ${file_list}.sorted > ${file_list}
			rm -rf $file_list.sorted ${full_list}
		done
		shlibs=
		dev_list=
		so_list=
		dbg_list=
		# Un splitted shared libraries go in the main package, not in the shlibs package
		# that is here only to allow loading ALL shared libraries at once
		rm -f ${file_list}
		touch ${file_list}
		
		#
		# Add a runtime dependency from the shlibs package to the splitted packages
		# so that loading xxxx-shlibs will load all shared libraries
		#
		split_dbg "Package '$bin_package' depends on '$all_shlibs_pkgs'"
		print_dependencies $bin_package rdepends "$needed_subpackages" $all_shlibs_pkgs
	fi

	# Here we must also intercept the locale sub-packages and split them further by language.
	# For all splitted packages, we must duplicate the metadata, but expanding the description 
	# by a line stating the contained library or the covered locale for locale we must keep
	# the non-langage-dependant files in the (forced) base locale package on which other depend
	
	# Finally generate the package
	build_package $bin_package $file_list $package $pkg
	echo ""
done

# Check that all files has been packaged
remaining_files=$(find . -type f | sed -e 's/^\.//')
if [ -n "$remaining_files" ]; then
	echo "#################################################################################################"
	echo "#"
	echo "# There are unpackaged files in package ${package_name}; please correct packaging directives"
	echo "#"
	echo "# Unpackaged files are:"
	for file in $remaining_files; do
		echo "# 	$file"
	done
	echo "#"
	echo "#################################################################################################"
	# Don't remove tempdir in case of error to help understand what happens
	exit 1
fi

# Now ensure that file paths in contents files are absolute (they were created with relative paths)
for cfile in $contentsTempDir/*.list; do
	mv $cfile $cfile.bak
	sed -e 's|^\([^/]\)|/\1|' $cfile.bak > $cfile
	rm -f $cfile.bak
done

# Finally automatically create dynamic libraries runtime dependencies
prefix="$(basename "${compiler_prefix}")"
readelf="${compiler_prefix}-readelf"
if [ ! -x "$( which "${readelf}" )" ]; then
    readelf="${prefix}-readelf"
fi
if [ ! -x "$( which "${readelf}" )" ]; then
    readelf="readelf"
fi
if [ ! -x "$( which "${readelf}" )" ]; then
    echo "Can't execute '${compiler_prefix}-readelf' nor '${prefix}-readelf' or 'readelf'"
    exit 1
fi

get_needed_libraries() {
	local bin_package="$1"
	local forced="$2"
	local package base subpkg
	(
		for file in $(cat $contentsTempDir/$bin_package.list); do
		    ${readelf} -d $installTempDir/$file 2> /dev/null	\
		    | grep -E '\(NEEDED\)' 2> /dev/null					\
		    | sed -r -e 's/^.*:[[:space:]]+\[(.*)\]$/\1/;'
		done | sort | uniq | while read lib; do
			# Look for the package providing this library
			grep -l "/$lib\$" $contentsTempDir/*.list $allContentsDir/*/*.list 2> /dev/null
		done | sed -e 's|^.*/||' -e 's/.list//' 
		package=${bin_package%_${package_suffix}}
		subpkg=${pkgkind[$package]}
		base=${package%-$subpkg}
		if [ "$subpkg" != "base" ]; then
			echo ${base}_${package_suffix}
			if [ -n "$forced" ]; then
				if [[ " base shlibs " =~ " $subpkg " ]]; then
					true # It's a base package, although it should not happen we can safely ignore the error
				else
					if [ -z "$subpkg" -o -z "$base" ]; then
						echo "Internal error on '$bin_package': base='$base' - subpkg='$subpkg'" >&2
					else
						cat $metadataTempDir/${base}_${package_suffix}.auto-depends | sed -e "s/_/-${subpkg}_/"
					fi
				fi
			fi
		fi
	) | sort | uniq
}

log "Generating ${package_name} automatic runtime dependencies"
for bin_package in $base_bin_packages $other_bin_packages; do
	get_needed_libraries $bin_package > $metadataTempDir/$bin_package.auto-depends
done
for bin_package in $forced_bin_packages; do
	get_needed_libraries $bin_package forced > $metadataTempDir/$bin_package.auto-depends
done

# Remove earlier definitions if any

if [ -s "$metadataDir/$recipe_id.manifest" ]; then
	(cd $packagesDir; rm -f $(cat "$metadataDir/$recipe_id.manifest"))
fi
rm -rf $contentsDir $tarballsDir $metadataDir

# Now install the generated packages
log "tar cf - -C $tempDir feeds | tar xf - -C $PLATFORM_LOC"
tar cf - -C $tempDir feeds | tar xf - -C $PLATFORM_LOC

do_buildstep_append
